@@ -24,11 +24,16 @@ __author__ = ('api.stephaniel@gmail.com (Stephanie Liu), '
               'api.jhartmann@gmail.com (Jochen Hartmann)')
 
 try:
-  from xml.etree import ElementTree
+  from xml.etree import cElementTree as ElementTree
 except ImportError:
-  from elementtree import ElementTree
+  try:
+    import cElementTree as ElementTree
+  except ImportError:
+    try:
+      from xml.etree import ElementTree
+    except ImportError:
+      from elementtree import ElementTree
 import os
-import urllib
 import atom
 import gdata
 import gdata.service
@@ -36,7 +41,9 @@ import gdata.youtube
 
 YOUTUBE_SERVER = 'gdata.youtube.com'
 YOUTUBE_SERVICE = 'youtube'
-YOUTUBE_SUPPORTED_UPLOAD_TYPES = ('mov', 'avi', 'wmv', 'mpg', 'quicktime')
+YOUTUBE_CLIENTLOGIN_AUTHENTICATION_URL = 'https://www.google.com/youtube/accounts/ClientLogin'
+YOUTUBE_SUPPORTED_UPLOAD_TYPES = ('mov', 'avi', 'wmv', 'mpg', 'quicktime',
+                                  'flv')
 YOUTUBE_QUERY_VALID_TIME_PARAMETERS = ('today', 'this_week', 'this_month',
                                        'all_time')
 YOUTUBE_QUERY_VALID_ORDERBY_PARAMETERS = ('published', 'viewCount', 'rating',
@@ -132,9 +139,10 @@ class YouTubeService(gdata.service.GDataService):
   def __init__(self, email=None, password=None, source=None,
                server=YOUTUBE_SERVER, additional_headers=None, client_id=None,
                developer_key=None):
+    self.additional_headers = {}
     if client_id is not None and developer_key is not None:
-      self.additional_headers = {'X-Gdata-Client': self.client_id,
-                                 'X-GData-Key': 'key=%s' % self.developer_key}
+      self.additional_headers = {'X-Gdata-Client': client_id,
+                                 'X-GData-Key': 'key=%s' % developer_key}
 
       gdata.service.GDataService.__init__(
           self, email=email, password=password, service=YOUTUBE_SERVICE, 
@@ -146,6 +154,7 @@ class YouTubeService(gdata.service.GDataService):
       gdata.service.GDataService.__init__(
           self, email=email, password=password, service=YOUTUBE_SERVICE,
           source=source, server=server, additional_headers=additional_headers)
+    self.auth_service_url = YOUTUBE_CLIENTLOGIN_AUTHENTICATION_URL
 
   def GetYouTubeVideoFeed(self, uri):
     """Retrieve a YouTubeVideoFeed.
@@ -566,6 +575,7 @@ class YouTubeService(gdata.service.GDataService):
             o video/quicktime
             o video/x-msvideo
             o video/mp4
+            o video/x-flv
 
     Returns:
       The newly created YouTubeVideoEntry if successful.
@@ -604,6 +614,7 @@ class YouTubeService(gdata.service.GDataService):
       mediasource = gdata.MediaSource()
       mediasource.setFile(filename_or_handle, content_type)
     elif hasattr(filename_or_handle, 'read'):
+      import StringIO
       if hasattr(filename_or_handle, 'seek'):
         filename_or_handle.seek(0)
       file_handle = StringIO.StringIO(filename_or_handle.read())
@@ -1354,6 +1365,9 @@ class YouTubeVideoQuery(gdata.service.Query):
     restriction: The restriction parameter identifies the IP address that
         should be used to filter videos that can only be played in specific
         countries.
+    location: A string of geo coordinates. Note that this is not used when the
+        search is performed but rather to filter the returned videos for ones
+        that match to the location entered.
   """
 
   def __init__(self, video_id=None, feed_type=None, text_query=None,
@@ -1390,8 +1404,9 @@ class YouTubeVideoQuery(gdata.service.Query):
 
   def _SetOrderBy(self, val):
     if val not in YOUTUBE_QUERY_VALID_ORDERBY_PARAMETERS:
-      raise YouTubeError('OrderBy must be one of: %s ' %
-                         ' '.join(YOUTUBE_QUERY_VALID_ORDERBY_PARAMETERS))
+      if val.startswith('relevance_lang_') is False:
+        raise YouTubeError('OrderBy must be one of: %s ' %
+                           ' '.join(YOUTUBE_QUERY_VALID_ORDERBY_PARAMETERS))
     self['orderby'] = val
 
   orderby = property(_GetOrderBy, _SetOrderBy,
@@ -1466,6 +1481,19 @@ class YouTubeVideoQuery(gdata.service.Query):
   restriction = property(_GetIPRestriction, _SetIPRestriction,
                          doc="""The restriction query parameter""")
 
+  def _GetLocation(self):
+    if 'location' in self:
+      return self['location']
+    else:
+      return None
+
+  def _SetLocation(self, val):
+    self['location'] = val
+
+  location = property(_GetLocation, _SetLocation,
+                      doc="""The location query parameter""")
+
+
 
 class YouTubeUserQuery(YouTubeVideoQuery):
 
